@@ -1,19 +1,24 @@
+from __future__ import annotations
+
 import dataclasses
 import inspect
-import typing
+
+from typing import Any, Dict, Generic, Mapping, Type, TypeVar
 
 from rest_framework.utils.model_meta import RelationInfo
 
 from rest_framework_dataclasses import typing_utils
+from rest_framework_dataclasses.types import Dataclass
 
-T = typing.TypeVar('T')
+T = TypeVar('T', bound=Dataclass)
+AnyT = TypeVar('AnyT')
 
 
 @dataclasses.dataclass
-class DataclassDefinition:
-    dataclass_type: type
-    fields: typing.Dict[str, dataclasses.Field]
-    field_types: typing.Dict[str, type]
+class DataclassDefinition(Generic[T]):
+    dataclass_type: Type[T]
+    fields: Dict[str, dataclasses.Field[Any]]
+    field_types: Dict[str, type]
 
 
 @dataclasses.dataclass
@@ -25,14 +30,12 @@ class TypeInfo:
     base_type: type
 
 
-def get_dataclass_definition(dataclass_type: type) -> DataclassDefinition:
+def get_dataclass_definition(dataclass_type: Type[T]) -> DataclassDefinition[T]:
     """
     Given a dataclass class, returns a dictionary mapping field names to
     `dataclasses.Field` instances describing all fields on the dataclass.
     """
-    # Resolve the typehint from the dataclass fields (which can be stringified, especially with PEP 563 nowadays) to
-    # actual type objects. Based on the discussion in https://stackoverflow.com/a/55938344.
-    types = typing.get_type_hints(dataclass_type)
+    types = typing_utils.get_resolved_type_hints(dataclass_type)
 
     # Disable PyCharm warning here, as it is wrong.
     # noinspection PyDataclass
@@ -48,7 +51,7 @@ def get_type_info(tp: type) -> TypeInfo:
     is_final = typing_utils.is_final_type(tp)
     if is_final:
         tp = typing_utils.get_final_type(tp)
-        if tp is typing.Any:
+        if tp is Any:
             # This used to be supported, but Python 3.10 raises in get_type_hints() if it encounters a plain Final hint.
             raise TypeError('Plain typing.Final is not valid as a type argument.')
 
@@ -67,9 +70,6 @@ def get_type_info(tp: type) -> TypeInfo:
     if typing_utils.is_type_variable(tp):
         tp = typing_utils.get_variable_type_substitute(tp)
 
-    if tp is typing.Any:
-        tp = None
-
     return TypeInfo(is_many, is_mapping, is_final, is_nullable, tp)
 
 
@@ -83,14 +83,18 @@ def get_relation_info(type_info: TypeInfo) -> RelationInfo:
         related_model=type_info.base_type,
         to_many=type_info.is_many,
         # as there's no foreign key, it also cannot reference a field on the referenced model
-        to_field=None,
+        to_field='',
         has_through_model=False,
         # we're never the model
         reverse=False
     )
 
 
-def lookup_type_in_mapping(mapping: typing.Dict[type, T], key: type) -> T:
+def lookup_type_in_mapping(mapping: Mapping[type, AnyT], key: type) -> AnyT:
+    # Allow all types, including special forms, to be used when they're present in the mapping
+    if key in mapping:
+        return mapping[key]
+
     try:
         # _SpecialForm types like Literal, NoReturn don't have an __mro__ attribute
         bases = inspect.getmro(key)
