@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 """
 Abstract away the runtime introspection on type hints behind an at least mostly-sane interface.
 
@@ -7,42 +9,39 @@ document what should be supported here, and to see what needs to be fixed for a
 
 Requirements for this module:
 
-  * Support for Python 3.7 - 3.9
+  * Support for Python 3.7 and above
   * Support both immediate and postponed evaluation of annotations (PEP 563, i.e. `from __future__ import annotations`)
   * Support collection generics from the typing module (i.e. typing.List[int])
   * Support standard collection generics (PEP 585, i.e. list[int])
+  * Support pipe operator syntax for unions (PEP 604, i.e. int | str)
 
 Note that there was some promising development in the `typing_inspect` module, but it is still marked experimental and
 development seems to have stalled. Maybe in the future?
 """
-import collections
+import collections.abc
+import sys
+import types
 import typing
 
 from .types import Final, Literal
 
-# typing._BaseGenericAlias was split-out from GenericAlias in Python 3.9
-if hasattr(typing, '_BaseGenericAlias'):
-    GenericAlias = typing._BaseGenericAlias
-else:
-    GenericAlias = typing._GenericAlias
-
 # types.UnionType was added in Python 3.10 for new PEP 604 pipe union syntax
-try:
-    from types import UnionType
-    UNION_TYPES = (typing.Union, UnionType)
-except ImportError:
+if sys.version_info >= (3, 10):
+    UNION_TYPES = (typing.Union, types.UnionType)
+else:
     UNION_TYPES = (typing.Union,)
 
-# Wrappers around typing.get_origin() and typing.get_args() for Python 3.7
-try:
+# typing.get_origin() and typing.get_args() were added in Python 3.8
+if sys.version_info >= (3, 8):
     get_origin = typing.get_origin
     get_args = typing.get_args
-except AttributeError:
+else:
+    # This is only for Python 3.7, so we don't have to worry about typing._BaseGenericAlias
     def get_origin(tp: type) -> type:
-        return tp.__origin__ if isinstance(tp, GenericAlias) else None
+        return tp.__origin__ if isinstance(tp, typing._GenericAlias) else None
 
-    def get_args(tp: type) -> type:
-        return tp.__args__ if isinstance(tp, GenericAlias) else ()
+    def get_args(tp: type) -> typing.Tuple:
+        return tp.__args__ if isinstance(tp, typing._GenericAlias) else ()
 
 # Some implementation notes:
 # * We detect if types are a generic by whether their origin is not None.
@@ -50,6 +49,32 @@ except AttributeError:
 #   if it's a type.
 
 
+def get_resolved_type_hints(tp: type) -> typing.Dict[str, type]:
+    """
+    Get the resolved type hints of an object.
+
+    Resolving the type hints means converting any stringified type hint into an actual type object. These can come from
+    either forward references (PEP 484), or postponed evaluation (PEP 563).
+    """
+    # typing.get_type_hints() does the heavy lifting for us, except when using PEP 585 generic types that contain a
+    # stringified type hint (see https://bugs.python.org/issue41370)
+    def _resolve_type(context_type: type, resolve_type: typing.Union[str, type]) -> type:
+        if isinstance(resolve_type, str):
+            globalsns = sys.modules[context_type.__module__].__dict__
+            return globalsns.get(resolve_type, resolve_type)
+        else:
+            return _resolve_type_hint(context_type, resolve_type)
+
+    def _resolve_type_hint(context_type: type, resolve_type: type) -> type:
+        if not hasattr(types, 'GenericAlias') or not isinstance(resolve_type, types.GenericAlias):
+            return resolve_type
+
+        args = tuple(_resolve_type(context_type, arg) for arg in resolve_type.__args__)
+        return typing.cast(type, types.GenericAlias(resolve_type.__origin__, args))
+
+    return {k: _resolve_type_hint(tp, v) for k, v in typing.get_type_hints(tp).items()}
+
+
 def is_iterable_type(tp: type) -> bool:
     """
     Test if the given type is iterable.
@@ -131,7 +156,7 @@ def is_optional_type(tp: type) -> bool:
     # * a generic
     # * with an origin of typing.Union (typing.Optional[int] reduces to typing.Union[int, None] at constructor time)
     # * with at least one argument that is NoneType
-    # * and at least one argumen that isn't NoneType
+    # * and at least one argument that isn't NoneType
     # except for Literals (sigh), which are just regular Literals with None as an allowed value.
     origin = get_origin(tp)
     args = get_args(tp)
@@ -158,7 +183,15 @@ def get_optional_type(tp: type) -> type:
         # Note that this doesn't remove `None` as a valid choice, but that's not a problem so far.
         return tp
 
-    return next(argument_type for argument_type in get_args(tp) if argument_type is not type(None))
+    remaining_arguments = tuple(arg for arg in get_args(tp) if arg is not type(None))
+    if len(remaining_arguments) == 1:
+        return remaining_arguments[0]
+    elif hasattr(types, 'UnionType') and isinstance(tp, types.UnionType):
+        # The types in the `types` module can't be instantiated in a generic way. Luckily only types.UnionType is
+        # relevant here, so handle it directly.
+        return typing.Union[remaining_arguments]  # type: ignore
+    else:
+        return get_origin(tp)[remaining_arguments]  # type: ignore
 
 
 def is_literal_type(tp: type) -> bool:
@@ -214,12 +247,14 @@ def is_type_variable(tp: type) -> bool:
     return isinstance(tp, typing.TypeVar)
 
 
-def get_variable_type_substitute(tp: typing.TypeVar) -> type:
+def get_variable_type_substitute(tp: type) -> type:
     """
     Get the substitute for a variable type.
     """
+    assert isinstance(tp, typing.TypeVar)
+
     if len(tp.__constraints__) > 0:
-        return typing.Union[tp.__constraints__]
+        return typing.Union[tp.__constraints__]  # type: ignore
     if tp.__bound__ is not None:
         return tp.__bound__
-    return typing.Any
+    return typing.Any  # type: ignore
